/*

Copyright (C) 2004 Christian Kreibich <christian@whoop.org>.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to
deal in the Software without restriction, including without limitation the
rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
sell copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies of the Software and its documentation and acknowledgment shall be
given in the documentation and software packages that this Software was
used.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

*/
#ifndef __libnd_phdl_h
#define __libnd_phdl_h

#include <libnd_phdl_types.h>
#include <libnd_phdl_exp.h>

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

typedef enum {
  LND_PHDL_STRUCT,
  LND_PHDL_BLOCK,
  LND_PHDL_CHAIN,
  LND_PHDL_UNION,
  LND_PHDL_ENUM,
  LND_PHDL_INT,
  LND_PHDL_HEX,
  LND_PHDL_STR,  
} LND_PHDL_FieldType;

struct lnd_phdl_headertempl
{
  char                   *name;
  LND_ProtocolLayer       layer;
  guint64                 magic;
  
  /* Dummy block template to contain all child field templates */
  LND_PHDL_FieldTempl    *root;
 
  void                   *demux_field;
};

struct lnd_phdl_fieldtempl
{
  LND_PHDL_FieldType      type;
  char                   *name;
  void                   *content;
};

typedef struct lnd_phdl_ftcommon
{
  guint                   use_count;
} LND_PHDL_FTCommon; 

typedef struct lnd_phdl_block
{
  LND_PHDL_FTCommon       common;
  guint                   use_count;
  GList                  *fields;
} LND_PHDL_Block;

typedef struct lnd_phdl_struct
{
  GList                  *fields;
} LND_PHDL_Struct;

typedef enum {
  LND_PHDL_CHAINEND_ITEMS,
  LND_PHDL_CHAINEND_BITS,
  LND_PHDL_CHAINEND_PATTERN,
} LND_PHDL_ChainEndType;

typedef struct lnd_phdl_chain_end
{
  LND_PHDL_ChainEndType   type;
  
  const LND_PHDL_NumExp  *exp;

  LND_PHDL_Field         *field;
  GList                  *fieldexps;
} LND_PHDL_ChainEnd;

typedef struct lnd_phdl_chain
{
  LND_PHDL_FTCommon         common;
  GList                    *fields;
  const LND_PHDL_ChainEnd  *end;
} LND_PHDL_Chain;

typedef struct lnd_phdl_union
{
  LND_PHDL_FTCommon       common;
  GList                  *fields;
  GList                  *bool_exps;
} LND_PHDL_Union;

typedef struct lnd_phdl_field_params
{
  guint                   unit;
  guint                   scale;
  char                   *sep;

  union {
    gint64                int_val;
    char                 *str_val;
  } val;

} LND_PHDL_FieldParams;

typedef struct lnd_phdl_numfield
{
  LND_PHDL_FTCommon       common;
  const LND_PHDL_NumExp  *length;
  gboolean                unsig;
  LND_PHDL_FieldParams   *params;
} LND_PHDL_NumField;

typedef struct lnd_phdl_strfield
{
  LND_PHDL_FTCommon       common;
  const LND_PHDL_NumExp  *length;
  char                   *end_marker;
  LND_PHDL_FieldParams   *params;
} LND_PHDL_StrField;

typedef struct lnd_phdl_enum
{
  LND_PHDL_FTCommon       common;
  GList                  *items;
  const LND_PHDL_NumExp  *length;
  gboolean                is_hex;

} LND_PHDL_Enum;

typedef struct lnd_phdl_enumitem
{
  char                   *label;
  gint64                  value;
} LND_PHDL_EnumItem;


typedef struct lnd_phdl_layerdef
{
  LND_ProtocolLayer       layer;
  guint64                 magic;
} LND_PHDL_LayerDef;


int                          libnd_phdl_parse_template(const char *filename);

void                         libnd_phdl_ctxt_push(void *data);
void                        *libnd_phdl_ctxt_pop(void);
void                        *libnd_phdl_ctxt_get(void);

LND_PHDL_HeaderTempl        *libnd_phdl_ht_new(const char *header_name,
							 LND_ProtocolLayer layer,
							 guint64 magic);

void                         libnd_phdl_ht_add_child(LND_PHDL_HeaderTempl *header,
							       LND_PHDL_FieldTempl *field);
GList                       *libnd_phdl_ht_get_children(LND_PHDL_HeaderTempl *header);
LND_Protocol                *libnd_phdl_ht_create_proto(LND_PHDL_HeaderTempl *header);
void                         libnd_phdl_ht_free(LND_PHDL_HeaderTempl *header);

LND_PHDL_FieldTempl         *libnd_phdl_ft_clone(const LND_PHDL_FieldTempl *ft,
						 const char *clone_name);
LND_PHDL_FieldTempl         *libnd_phdl_ft_struct_new(const char *name);
LND_PHDL_FieldTempl         *libnd_phdl_ft_block_new(const char *name);
LND_PHDL_FieldTempl         *libnd_phdl_ft_chain_new(const char *name);
void                         libnd_phdl_ft_chain_set_end(LND_PHDL_FieldTempl *chain, const LND_PHDL_ChainEnd *end);

LND_PHDL_ChainEnd           *libnd_phdl_ft_chainend_new_items(LND_PHDL_NumExp *exp);
LND_PHDL_ChainEnd           *libnd_phdl_ft_chainend_new_length(LND_PHDL_NumExp *exp);
LND_PHDL_ChainEnd           *libnd_phdl_ft_chainend_new_pattern(LND_PHDL_FieldTempl *pattern);
void                         libnd_phdl_ft_chainend_free(LND_PHDL_ChainEnd *end);

LND_PHDL_FieldTempl         *libnd_phdl_ft_union_new(const char *name);
void                         libnd_phdl_ft_union_add_field(LND_PHDL_FieldTempl *un,
							LND_PHDL_FieldTempl *field,
							LND_PHDL_BoolExp *exp);

LND_PHDL_FieldTempl         *libnd_phdl_ft_enum_new(const char *name,
						 const LND_PHDL_NumExp *exp);
void                         libnd_phdl_ft_enum_set_hex_format(LND_PHDL_FieldTempl *field, gboolean is_hex);
void                         libnd_phdl_ft_enum_add_item(LND_PHDL_FieldTempl *field,
						      LND_PHDL_EnumItem *item);
LND_PHDL_EnumItem           *libnd_phdl_ft_enumitem_new(const char *name, gint64 value);
void                         libnd_phdl_ft_enumitem_free(LND_PHDL_EnumItem *item);

LND_PHDL_FieldParams        *libnd_phdl_ft_fieldparams_new(void);
LND_PHDL_FieldParams        *libnd_phdl_ft_fieldparams_copy(LND_PHDL_FieldParams *params);
void                         libnd_phdl_ft_fieldparams_free(LND_PHDL_FieldParams *params);

LND_PHDL_FieldTempl         *libnd_phdl_ft_int_new(const char *name, const LND_PHDL_NumExp *exp); 
LND_PHDL_FieldTempl         *libnd_phdl_ft_hex_new(const char *name, const LND_PHDL_NumExp *exp);
LND_PHDL_FieldTempl         *libnd_phdl_ft_str_fixed_new(const char *name, const LND_PHDL_NumExp *exp);
LND_PHDL_FieldTempl         *libnd_phdl_ft_str_variable_new(const char *name, const char *end_marker);

gboolean                     libnd_phdl_ft_set_params(LND_PHDL_FieldTempl *templ,
						      LND_PHDL_FieldParams *params);
const LND_PHDL_FieldParams  *libnd_phdl_ft_get_params(LND_PHDL_FieldTempl *templ);
void                         libnd_phdl_ft_add_child(LND_PHDL_FieldTempl *parent,
						     LND_PHDL_FieldTempl *child);
void                         libnd_phdl_ft_free(LND_PHDL_FieldTempl *field);


LND_Protocol                *libnd_phdl_get_header_from_proto(LND_PHDL_HeaderTempl *header);
LND_PHDL_HeaderTempl        *libnd_phdl_get_proto_from_header(LND_Protocol *proto);

void                         libnd_phdl_ht_ctxt_set(LND_PHDL_HeaderTempl *header);
LND_PHDL_HeaderTempl        *libnd_phdl_ht_ctxt_get(void);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* __libnd_phdl_h */
