/*

Copyright (C) 2004 Christian Kreibich <christian@whoop.org>.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to
deal in the Software without restriction, including without limitation the
rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
sell copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies of the Software and its documentation and acknowledgment shall be
given in the documentation and software packages that this Software was
used.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

*/
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include "libnd_phdl.h"
#include "libnd_phdl_reg.h"

const char *
name(void)
{
  return "PHDL";
}

const char *
author(void)
{
  return "Christian Kreibich, <christian.kreibich-AT-cl.cam.ac.uk>";
}

const char *
version(void)
{
  return VERSION;
}

gboolean
init(void)
{
  libnd_phdl_reg_init();
  return TRUE;
}

static void
phdl_help(void)
{
  printf("PHDL interpreter plugin\n"
	 "USAGE: lndtool -r phdl <templ1> [<templ2> <templ3> ...]\n"
	 "\n"
	 "  --help, -h, -?           This message.\n");
}

gboolean
run(LND_Trace *unused, LND_PluginArgs *args)
{
  int i;

  if (!args)
    return FALSE;

  if (args->argc == 0)
    {
      printf("Please provide one or more templates to parse.\n");
      phdl_help();
      return FALSE;
    }

  for (i = 0; i < args->argc; i++)
    {
      if (!strcmp(args->argv[i], "-h")     ||
	  !strcmp(args->argv[i], "--help") ||
	  !strcmp(args->argv[i], "-?"))
	{
	  phdl_help();
	  return FALSE;
	}
      else
	{
	  printf("Parsing template file %s\n", args->argv[i]);
	  libnd_phdl_parse_template(args->argv[i]);
	}
    }
  
  return TRUE;
  TOUCH(unused);
}
