/*

Copyright (C) 2004 Christian Kreibich <christian@whoop.org>.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to
deal in the Software without restriction, including without limitation the
rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
sell copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies of the Software and its documentation and acknowledgment shall be
given in the documentation and software packages that this Software was
used.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

*/
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>

#include "libnd_phdl_reg.h"

static LND_Registry *header_reg;
static LND_Registry *struct_reg;


void
libnd_phdl_reg_init(void)
{
  static gboolean deja_vu = FALSE;

  if (! deja_vu)
    {
      header_reg = libnd_reg_new("header_templates");
      struct_reg = libnd_reg_new("struct_templates");
      deja_vu = TRUE;
    }
}


void                    
libnd_phdl_htreg_add(const LND_PHDL_HeaderTempl *header)
{
  if (!header)
    return;

  D(("Registering protocol header '%s'\n", header->name));
  libnd_reg_set_data(header_reg, header->name, (void *) header);
}


LND_PHDL_HeaderTempl *
libnd_phdl_htreg_get(const char *templ_name)
{
  return libnd_reg_get_data(header_reg, templ_name);
}


void                    
libnd_phdl_structreg_add(const LND_PHDL_FieldTempl *ft)
{
  if (!ft)
    return;

  D(("Adding structure '%s' to repository.\n", ft->name));
  libnd_reg_set_data(struct_reg, ft->name, (void *) ft);
}


static void
phdl_structreg_cb(const char *name, void *data, void *user_data)
{
  D(("Item: %s --> %p\n", name, data));
}

LND_PHDL_FieldTempl *
libnd_phdl_structreg_get(const char *templ_name)
{
  libnd_reg_foreach(struct_reg, phdl_structreg_cb, NULL);

  D(("Looking up structure '%s' in repository.\n", templ_name));
  return libnd_reg_get_data(struct_reg, templ_name);
}
