#!/usr/bin/perl -w

# CVS wrapper script that maintains a nice ChangeLog for us.
# Heavily hacked & cleaned up, originally based upon a script
# that was once used in the Enlightenment project.


use strict;
use FileHandle;


# Prototypes
#_______________________________________________________________________

sub create_changelog;
sub validate_commit_message;
sub create_commit_message;
sub setup_username_translation;
sub update_changelog;
sub check_parameters;


# Globals
#_______________________________________________________________________
my %names;
my ($date, $author);

# Formats
#_______________________________________________________________________
format ENTRY =
@<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<< @>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
$date, $author
.


# Subroutines
#_______________________________________________________________________

sub setup_username_translation {
  
  $names{"cpk25"} = "Christian <christian\@whoop.org>";
  $names{"kreibich"} = "Christian <christian\@whoop.org>";
  $names{"christian"} = "Christian <christian\@whoop.org>";
}



sub create_changelog {
  
  my $cl = "ChangeLog";
  my $num_lines;
  my $wc;
  my @lines;
  my $line;
  
  print "Updating the ChangeLog with your entry.\n";
  
  if (open CHANGELOG, $cl)
    {
      @lines = <CHANGELOG>;
      close CHANGELOG;

      shift (@lines);
      shift (@lines);
    }  
  

  open CHANGELOG, ">$cl";
  print CHANGELOG <<eof;
libnetdude plugin essentials changelog
========================================================================

eof
  close CHANGELOG;

  `cat CommitLog >> $cl`;
  
  open CHANGELOG, ">>$cl";
  print CHANGELOG "\n------------------------------------------------------------------------\n";
  foreach $line (@lines)
    {
      print CHANGELOG $line;
    }  
  close CHANGELOG;

  # We've modified the change log, copy it over to the website:
  print "Updating ChangeLog on netdude.sf.net.\n";
  system('scp ChangeLog cpk@shell.sourceforge.net:/home/groups/n/ne/netdude/htdocs/ChangeLog-plugins');
}


sub validate_commit_message {
  
  my @lines;
  
  open COMMITLOG, "CommitLog";
  @lines = <COMMITLOG>;
  close COMMITLOG;
  
  if($#lines < 1)
    {
      print "Message too short -- aborting commit.\n";
      exit(0);
    }
}


sub create_commit_message {
  
  print "Please create a log message for the ChangeLog.\n";
  
  open COMMITLOG, ">CommitLog";

  COMMITLOG->format_name("ENTRY");
  $date   = `date`;
  $author = $names{$ENV{USER}};
  write COMMITLOG;
  print COMMITLOG "\n";
  close COMMITLOG;
  
  if($ENV{EDITOR})
    {
      system("$ENV{EDITOR} CommitLog");
    }
  else
    {
      system("vi CommitLog");
    }
}


sub update_changelog {
  
  my @ARGV2;
  
  @ARGV2 = @ARGV;
  $ARGV2[0] = "update";
  
  print "Force updating ChangeLog\n";
  unlink "ChangeLog";
  system("cvs -z5 update ChangeLog");
}


sub check_parameters {

  if (scalar(@ARGV) < 1)
    {
      print "USAGE: cvs.pl <cvs comands> <files>\n";
      exit (0);
    }
}

# Main Program
#_______________________________________________________________________

check_parameters();
setup_username_translation();

if (($ARGV[0] =~ /com/) || ($ARGV[0] =~ /ci/))
  {
    my @ARGV2;
    my $message;

    create_commit_message();
    validate_commit_message();
    update_changelog();

    $message = `cat CommitLog`;
    $ARGV[0] .= " -m \"$message\"";
    @ARGV2 = @ARGV;
    $ARGV2[0] = "update";

    print "Updating the files you are committing.\n";
    system("cvs -z5 @ARGV2 2>&1 |tee errors");
    
    if (open ERRORS, "errors")
      {
	while(<ERRORS>)
	  {
	    if (/conflicts during merge/)
	      {
		print "There are one or more conflicts,\n" .
		  "Please resolve and try again.\n";
		unlink "errors" if(-f "errors");
		exit(0);
	      }
	  }
	close ERRORS;
      }
    
    unlink "errors" if(-f "errors");
    
    create_changelog();
    
    if($#ARGV >= 1)
      {
	my $found;
	
	$found = 0;
	foreach(@ARGV) {
	  if(/ChangeLog$/)
	    {
	      $found = 1;
	    }
        }
	push @ARGV, "ChangeLog" if(!$found);
      }
  }

print "cvs -z5 @ARGV\n";
system("cvs -z5 @ARGV");

print "Finished.\n"
